%% BMI_Master.m: Perform all calculations and generate all figures

%% 1. Load data
load('BRFSS_BMI.csv')
load('NHANES_DM.csv')
load('NHANES_SR.csv')
load('NU.csv')

% Remove NU data for years where there are less than 1000 BMI measurements
y0 = min(NU(:,1));
yf = max(NU(:,1));

for t = y0:yf
    if nnz(NU(:,1) == t) < 1000
        NU(NU(:,1) == t,:) = [];
    end
end

%% 2. Make Figure 1
T = unique(BRFSS_BMI(:,1));

% Compute mean, standard deviation, skewness from BMI data
moments = zeros(length(T),3);

for t = 1:length(T)
    idx = BRFSS_BMI(:,1) == T(t);
    y = BRFSS_BMI(idx,2);
    
    moments(t,1) = mean(y);
    moments(t,2) = std(y);
    moments(t,3) = (mean(y.^3) - 3*moments(t,1)*moments(t,2)^2 -...
        moments(t,1)^3)/moments(t,2)^3;
end

clearvars idx y t

% Plot Mean, SD, Skewness versus Time
figure
set(gcf, 'units', 'centimeters', 'pos', [2, 2, 17, 4])

for i=1:3
    subplot(1,4,i)
    
    if i == 1
        ylab = '(a) Mean';
        pos = [1, 1, 3, 2.25];
    elseif i == 2
        ylab = '(b) SD';
        pos = [5,1,3,2.25];
    elseif i==3
        ylab = '(c) Skewness';
        pos = [9,1,3,2.25];
    end
    
    plot(T, moments(:,i), '.k')
    hold on
    X = [T,ones(length(T),1)];
    b = (X'*X)\X'*moments(:,i);
    plot(T, b(2) + b(1)*T,'-k')
    set(gca, 'units', 'centimeters', 'pos', pos)
    
    xlabel('Year ($t$)', 'Interpreter','Latex')
    title(ylab)
end

% Plot distributions in 1991 and 2011
subplot(1,4,4)

% Generate histograms for 1991, 2011
edges = 0:1:100;
N1991 = histcounts(BRFSS_BMI(BRFSS_BMI(:,1)==1991,2), edges, ...
    'Normalization', 'pdf');
N2011 = histcounts(BRFSS_BMI(BRFSS_BMI(:,1)==2011,2), edges, ...
    'Normalization', 'pdf');

% Smooth raw data using convolution
c = exp(-0.5*(-4:1:4).^2)/sqrt(2*pi);
c = c/sum(c);

p1991 = conv(N1991, c, 'same');
p2011 = conv(N2011, c, 'same');

% Plot
edges = 0.5*(edges(1:end-1)+edges(2:end));
plot(edges, p1991, '-r', edges, N1991, '^r', ...
    edges, p2011, '--b', edges, N2011, '.b')

% Plot options
pos = [13,1,3,2.25];
set(gca, 'units', 'centimeters', 'pos', pos)

set(gca,'XTick', 20:10:50)
set(gca,'XTickLabel',{'20','30','40','50'})

axis([15,55,0,0.12])

xlabel('BMI', 'Interpreter','Latex')

ylab = '(d) Dist''n';
title(ylab)

% save2pdf('Fig1',gcf,600)
saveas(gcf,'Fig1.fig')

close

clearvars b c edges i moments N1991 N2011 p1991 p2011 pos ylab X

%% 3. Individual Level (Fig. 2, Table S1 - see NHind and NUind)
% Characterize a and b using micro-level (Delta BMI) data
NU = sortrows( NU(NU(:,3)>0,:) );
NHANES_SR = sortrows( NHANES_SR(NHANES_SR(:,3)>0,:) );

[NHind, NUind] = fitAB(1, 1, NHANES_SR, NU, 'Fig2');

save('BMIresults.mat', 'NHind', 'NUind')

% Repeat for all gender and age groups
% Male, Female
% 18-28, 28-38, 38-48, 48-58, 58-68, 68-78, 78+
Nbreakdown = zeros(7,2,2);
xast = zeros(7,2,2);
ki = zeros(7,2,2);
kb = zeros(7,2,2);

for i = 1:2
    for j = 1:7
        if i == 1
            saveName = 'Fig2_Female';
        else
            saveName = 'Fig2_Male';
        end
        
        NHANES_temp = NHANES_SR(NHANES_SR(:,5)==i-1,:);
        NU_temp = NU(NU(:,5)==i-1,:);
        
        if j == 1
            saveName = strcat(saveName,'_18-28');
            NHANES_temp = NHANES_temp(NHANES_temp(:,4) >= 18 ...
                & NHANES_temp(:,4) < 28,:);
            NU_temp = NU_temp(NU_temp(:,4) >= 18 & NU_temp(:,4) < 28,:);
        elseif j == 2
            saveName = strcat(saveName,'_28-38');
            NHANES_temp = NHANES_temp(NHANES_temp(:,4) >= 28 ...
                & NHANES_temp(:,4) < 38,:);
            NU_temp = NU_temp(NU_temp(:,4) >= 28 & NU_temp(:,4) < 38,:);
        elseif j == 3
            saveName = strcat(saveName,'_38-48');
            NHANES_temp = NHANES_temp(NHANES_temp(:,4) >= 38 ...
                & NHANES_temp(:,4) < 48,:);
            NU_temp = NU_temp(NU_temp(:,4) >= 38 & NU_temp(:,4) < 48,:);
        elseif j == 4
            saveName = strcat(saveName,'_48-58');
            NHANES_temp = NHANES_temp(NHANES_temp(:,4) >= 48 ...
                & NHANES_temp(:,4) < 58,:);
            NU_temp = NU_temp(NU_temp(:,4) >= 48 & NU_temp(:,4) < 58,:);
        elseif j == 5
            saveName = strcat(saveName,'_58-68');
            NHANES_temp = NHANES_temp(NHANES_temp(:,4) >= 58 ...
                & NHANES_temp(:,4) < 68,:);
            NU_temp = NU_temp(NU_temp(:,4) >= 58 & NU_temp(:,4) < 68,:);
        elseif j == 6
            saveName = strcat(saveName,'_68-78');
            NHANES_temp = NHANES_temp(NHANES_temp(:,4) >= 68 ...
                & NHANES_temp(:,4) < 78,:);
            NU_temp = NU_temp(NU_temp(:,4) >= 68 & NU_temp(:,4) < 78,:);
        else
            saveName = strcat(saveName,'_78');
            NHANES_temp = NHANES_temp(NHANES_temp(:,4) >= 78,:);
            NU_temp = NU_temp(NU_temp(:,4) >= 78,:);
        end
        
        Nbreakdown(j,i,1) = size(NHANES_temp,1);
        Nbreakdown(j,i,2) = size(NU_temp,1);
        
        [NHind_temp, NUind_temp] = fitAB(1, 1, NHANES_temp, NU_temp, saveName);
        
        xast(j,i,1) = NHind_temp(1,7);
        xast(j,i,2) = NUind_temp(1,7);
        ki(j,i,1) = NHind_temp(1,5);
        ki(j,i,2) = NUind_temp(1,5);
        kb(j,i,1) = NHind_temp(1,3);
        kb(j,i,2) = NUind_temp(1,3);
    end
end

% Plot parameters
% Left panels: female, right panels: male
% Blue: NHANES, Red: NU
figure
set(gcf, 'units', 'centimeters', 'pos', [2, 2, 17, 13])

subplot(2,2,1)
plot(18:10:78+5,xast(:,1,1),'-bo',18:10:78+5,xast(:,1,2),'--sr')
% title('(a) Female')
% xlabel('Age','Interpreter','Latex')
ylabel('$x^*$','Interpreter','Latex')
axis([10,80,20,35])

subplot(2,2,3)
plot(18:10:78+5, ki(:,1,1)./kb(:,1,1).^2, '-bo',...
    18:10:78+5, ki(:,1,2)./kb(:,1,2).^2, '--sr')
xlabel('Age','Interpreter','Latex')
ylabel('$k_I/k_b$','Interpreter','Latex')
axis([10,80,0,60])

subplot(2,2,2)
plot(18:10:78+5,xast(:,2,1),'-bo',18:10:78+5,xast(:,2,2),'--sr')
% ylabel('$x^*$','Interpreter','Latex')
axis([10,80,20,35])

subplot(2,2,4)
plot(18:10:78+5, ki(:,2,1)./kb(:,2,1).^2, '-bo',...
    18:10:78+5, ki(:,2,2)./kb(:,2,2).^2, '--sr')
xlabel('Age','Interpreter','Latex')
% ylabel('$k_I/k_b$','Interpreter','Latex')
axis([10,80,0,60])

saveas(gcf,'Fig2_param','fig')
% save2pdf('Fig2_param',gcf,600)

% Repeat, aggregating all years together in order to generate supp.
% versiton of Fig. 2
NHANES_temp = NHANES_SR;
NHANES_temp(:,1) = 2011;

NU_temp = NU;
NU_temp(:,1) = 2011;

fitAB(1, 1, NHANES_temp, NU_temp, 'Fig2_agg');


%% 4. Population Level (Fig. 3, Table 1, Fig. S1-S2)
% Fit non-social, social, log-normal, skew-normal models to empirical BMI
% distributions.
%
% See NHpop, NUpop, BRpop for Table 1

% Bin edges for empirical BMI distribution
x = 10:1:100;

[NHpop, NUpop, BRpop, BRfit] = fitBMIdistn(x, NHANES_DM, NU(:,1:2),...
    BRFSS_BMI);

save('BMIresults.mat', 'NHpop', 'NUpop', 'BRpop', 'BRfit', '-append')

% Make Fig. S3
figure
set(gcf, 'units', 'centimeters', 'pos', [2, 2, 8, 6])
plot(BRpop(:,1),BRpop(:,10),'o-b')
ylabel('$k_0=k_I/k_b$', 'Interpreter', 'Latex')
xlabel('Year $(t)$', 'Interpreter', 'Latex')
axis([1985, 2015, 10, 20])

saveas(gcf,'FigS2.fig')
% save2pdf('FigS2',gcf,600)
close

%% 5. Solve PDE (BRFSS only)
% - Assume x^*, k_I, k_b all vary linearly
% - Find coefficients for linear relationships that yield optimal fit when
% solution to PDE is compared to BRFSS distributions.
% - Plot results

dpdt(BRpop(:,1), BRfit, BRFSS_BMI)