%% fitAB.m: Compute coefficients for a(x) and b(x) using micro-level data.
%
% Model:
% Non-social model:
% a(x) = k_I (x^* - x)
% b(x) = sqrt(k_b) x
%
% Social model:
% a(x) = k_I (x^* - x) + k_S sum_j [Aij(xj - xi) -0.5 dAij/dxi (xj - xi)^2]
% b(x) = sqrt(k_b) x
%
% Aij = 1/N phi( (xj-x)/sigma ), N = number of individuals
%
% Data:
% A(x) = mean( Delta x/Delta t ), and
% B(x) = sqrt( variance( Delta x ) / Delta t )
% where Delta x is the average of the change in BMI over time interval
% Delta t for all individuals whose initial BMI is within dx/2 of x
%
% To compute the coefficients we regress A(x) and B(x) (which are computed
% from the data) on the terms in a(x) and b(x).
%
% Inputs:
%   - dx
%   - BMI1 (NHANES), BMI2 (NU):
%       - 3 column array.
%       - column 1: year t (sorted in ascending order)
%       - column 2: BMI measurement at time t
%       - column 3: BMI measurement at time t+1
%
% Outputs:
%   - out1 (NHANES), out2 (BRFSS)
%       - out#(1,:) = non-social model
%       - out#(2,:) = social model
%       - out#(:,1:2) = RMS Error a, RMS Error b
%       - out#(:,3:4) = sqrtt(k_b), width of 95% CI, i.e. w( sqrt(k_b) )
%       - out#(:,5:12) = k_I, w(k_I), x^*, w(x^*), k_S, w(k_S), sigma,
%       w(sigma)
%
% Global Variables
%   - A: see above for def'n of A(x)
%   - X: [BMI year 1, BMI year 2]
%   - x, dXij: grid for fine scale, used for interpolation
%   - p: histogram of initial BMIs
%   - idxT: indices for surveys by year

function [out1, out2] = fitAB(dx, dt, BMI1, BMI2, saveName)
global A X x dXij p idxT
% Grid for interpolation calcs, see computation of A, B, and obj_ab
dx0 = dx/10;
xmin = 10;
xmax = 100;

x = floor(xmin):dx0:ceil(xmax);
dXij = kron(x,ones(length(x),1));
dXij = dXij - dXij';

% Years of surveys
T1 = sort(unique(BMI1(:,1)));
T2 = sort(unique(BMI2(:,1)));

% Fig. 2: 2011 A and B
figure
set(gcf, 'units', 'centimeters', 'pos', [2, 2, 18.3, 8])
hold on;
box on;

% i = 1: BMI1, NHANES
% i = 2: BMI2, NU
for i = 1:2
    % BMI1 (NHANES), BMI2 (NU) data
    if i == 1
        BMI = BMI1(BMI1(:,3)>0,:);
        T = T1;
    elseif i == 2
        BMI = BMI2(BMI2(:,3)>0,:);
        T = T2;
    end
    
    % Indices for BMIs by year
    idxT = zeros(1,length(T)+1);
    
    for t = 1:length(T)
        idxT(t+1) = find(T(t)==BMI(:,1), 1, 'last');
    end
    
    % p: histograms by year
    p = zeros(length(x), length(T));
    
    for t = 1:length(T)
        p(:,t) = histcounts(BMI(idxT(t)+1:idxT(t+1),2), ...
            [x-dx0/2, x(end)+dx0/2], 'Normalization', 'pdf');
    end
    
    % Delete column indicating year
    BMI = BMI(:,2:3);
    
    % Temporary output
    out = zeros(2,12);
    
    %% Compute A, B, X:
    A = zeros(length(BMI),1);
    B = zeros(length(BMI),1);
    X = BMI(:,1);
    
    for t = 1:length(T)
        % Temporary BMIs, A, B
        BMIt = BMI(idxT(t)+1:idxT(t+1),:);

        Atemp = zeros(length(x),1);
        Btemp = zeros(length(x),1);
        
        n = 0;
        
        % Compute A and B on a grid and then use linear interpolation
        for j = 1:length(x)
            idxA = abs(BMIt(:,1) - x(j)) < dx/2;
            dBMIt = BMIt(idxA,2)-BMIt(idxA,1);
            
            n = n + length(dBMIt);
            
            Atemp(j) = mean( dBMIt )/dt;
            Btemp(j) = sqrt( mean( dBMIt.^2 )/dt - dt*Atemp(j).^2 );
        end
        
        A(idxT(t)+1:idxT(t+1)) = interp1(x, Atemp, BMIt(:,1), 'linear',...
            'extrap');
        B(idxT(t)+1:idxT(t+1)) = interp1(x, Btemp, BMIt(:,1), 'linear',...
            'extrap');
        
        if T(t) == 2011
            if i == 1
                Atemp1 = Atemp(1:10:length(x));
                Btemp1 = Btemp(1:10:length(x));
            else
                Atemp2 = Atemp(1:10:length(x));
                Btemp2 = Btemp(1:10:length(x));
            end
        end
    end
    
    %% Estimate parameters for non-social model
    b = (X'*X)\(X'*B);
    r = B - X*b;
    out(1,2) = sqrt( mean( r.^2 ) );
    
    bint = 1.96*sqrt( out(1,2)^2/(X'*X) );
    bint = [b-bint, b+bint];
    out(1,3:4) = [b, diff(bint)];
        
    Xtemp = [-X, ones(length(X),1)];
    b = (Xtemp'*Xtemp)\(Xtemp'*A);
    r = A - Xtemp*b;
    out(1,1) = sqrt( mean( r.^2 ) );
    
    V = out(1,1)^2*( (Xtemp'*Xtemp)\eye(2) );
    bint = [b - 1.96*sqrt(diag(V)), b + 1.96*sqrt(diag(V))];
    out(1,[5,7]) = [b(1), b(2)/b(1)];
    out(1,6) = diff(bint(1,:));
    out(1,8) = 2*1.96*sqrt(V(2,2)/b(1)^2 + V(1,1)*b(2)^2/b(1)^4 ...
        - 2*V(1,2)^2*b(2)/b(1)^3);
    
    %% Estimate parameters for social model
    out(2,2:4) = out(1,2:4);
    out(2,11) = fminsearch(@obj_ab, 2);
    
    [S, Xsoc, b] = obj_ab(out(2,11));
    
    out(2,1) = S;
    out(2,5:2:9) = [b(1), b(2)/b(1), b(3)];
    
    % Compute Confidence intervals for NLLS
    h = out(2,11)/10;
    [~, XSoc2] = obj_ab(out(2,11)+h);
    [~, XSoc1] = obj_ab(out(2,11)-h);
    
    dXSoc2 = (XSoc2 - XSoc1)/2/h;
    
    d = 1;
    
    while d >= 1e-9
        h = h/10;
        [~, XSoc2] = obj_ab(out(2,11)+h);
        [~, XSoc1] = obj_ab(out(2,11)-h);
        
        dXSoc1 = dXSoc2;
        dXSoc2 = (XSoc2 - XSoc1)/2/h;
        
        d = sqrt( mean( (dXSoc1 - dXSoc2).^2) );
    end
    
    J = [b(2)-X, b(1)*ones(length(X),1), Xsoc, b(3)*dXSoc2];
    out(2,6:2:12) = 1.96*2*diag( S^2*(J'*J)\eye(4) );
    
    % Record results
    if i == 1
        out1 = out;
%         X1 = X;
%         A1 = A;
%         B1 = B;
%         Xsoc1 = Xsoc;
    else
        out2 = out;
%         X2 = X;
%         A2 = A;
%         B2 = B;
%         Xsoc2 = Xsoc;
    end
    
    %% Fig. 2 (NHANES left panel, do first)
    subplot(1,2,-i+3)
    
    % Indices of measurements from 2011
    idx2011 = find(T==2011);
    idx2011 = idxT(idx2011)+1:idxT(idx2011+1);
    
    % X and Xsoc from 2011
    X_Xsoc = [X(idx2011), Xsoc(idx2011)];
    X_Xsoc = sortrows(X_Xsoc);
    
    % Plot Data
    if i == 1
        plot(x(1:10:length(x)), Btemp1, 'Marker', '^',...
            'MarkerEdgeColor', 'r', 'LineStyle', 'none', 'MarkerSize', 3)
        hold on; box on;
        scatter(x(1:10:length(x)),Atemp1,'.b')
    else
        plot(x(1:10:length(x)), Btemp2, 'Marker', '^',...
            'MarkerEdgeColor', 'r', 'LineStyle', 'none', 'MarkerSize', 3)
        hold on; box on;
        scatter(x(1:10:length(x)),Atemp2,'.b')
    end
   
    % Plot Line of Best Fit
    plot(x,zeros(length(x),1),'--k')
    plot(X_Xsoc(:,1), out(2,5)*(out(2,7) -X_Xsoc(:,1)) +...
        out(2,9)*X_Xsoc(:,2), '-k')
    plot(X_Xsoc(:,1), out(1,3)*X_Xsoc(:,1), '-k')
    
    % Axes properties, annotation, save figure
    set(gca,'XTick', 20:10:50)
    xlabel('BMI $(x)$', 'Interpreter', 'Latex', 'FontSize', 12)
    set(gca,'XTickLabel',{'20','30','40','50'})
    
    if i == 1
        axis([15, 55, -5, 7.5])
        set(gca,'YTick', -5:5:10)
        set(gca,'YTickLabel',{'-5','0','5','10'})
        set(gca, 'units', 'centimeters', 'pos', [9.65, 1.25, 8.4, 6])
        
        annotation('textbox', 'Units', 'centimeters',...
            'Position', [10.15, 5.5, 4, 1],...
            'String', 'diffusion proportional to BMI',...
            'FontWeight', 'bold', 'Color', 'red', 'LineStyle', 'none');
        
        annotation('arrow', 'Units', 'centimeters',...
            'X', [11.5, 12.5], 'Y', [2.5, 2.5], 'Color', 'blue',...
            'LineWidth',2)
        annotation('arrow', 'Units', 'centimeters',...
            'X', [14, 13], 'Y', [2.5, 2.5], 'Color', 'blue',...
            'LineWidth',2)
        annotation('textbox', 'Units', 'centimeters',...
            'Position', [11.5, 1.5, 2, 1], 'String', 'drift',...
            'FontWeight', 'bold', 'Color', 'blue', 'LineStyle', 'none');
        annotation('textbox', 'Units', 'centimeters',...
            'Position', [13, 1.5, 2, 1], 'String', 'drift',...
            'FontWeight', 'bold', 'Color', 'blue', 'LineStyle', 'none');
        
        title('\textbf{NHANES}', 'Interpreter', 'Latex', 'FontSize', 12)
    else
        axis([15, 55, -3, 6])
        set(gca,'YTick', -2:2:6)
        set(gca,'YTickLabel',{'-2','0','2','4','6'})
        set(gca, 'units', 'centimeters', 'pos', [0.5, 1.25, 8.4, 6])
        
        annotation('textbox', 'Units', 'centimeters',...
            'Position', [1, 5.5, 4, 1],...
            'String', 'diffusion proportional to BMI',...
            'FontWeight', 'bold', 'Color', 'red', 'LineStyle', 'none');
        
        annotation('arrow', 'Units', 'centimeters',...
            'X', [1.75, 2.75], 'Y', [2.5, 2.5], 'Color', 'blue',...
            'LineWidth',2)
        annotation('arrow', 'Units', 'centimeters',...
            'X', [4.25, 3.25], 'Y', [2.5, 2.5], 'Color', 'blue',...
            'LineWidth',2)
        annotation('textbox', 'Units', 'centimeters',...
            'Position', [1.75, 1.5, 2, 1], 'String', 'drift',...
            'FontWeight', 'bold', 'Color', 'blue', 'LineStyle', 'none');
        annotation('textbox', 'Units', 'centimeters',...
            'Position', [3.25, 1.5, 2, 1], 'String', 'drift',...
            'FontWeight', 'bold', 'Color', 'blue', 'LineStyle', 'none');
        
        title('\textbf{NU}', 'Interpreter', 'Latex', 'FontSize', 12)
        
        saveas(gcf,saveName,'fig')
%         save2pdf(saveName,gcf,600)
        close
    end
end
end

%% Objective function for NLLS param. estimation of social model
function [S, Xsoc, b, r] = obj_ab(s)
global A X x dXij p idxT

Xsoc = zeros(length(X),1);

% For each xi, xj in X: approx.
%   sum_j [Aij(xj - xi) - 0.5 dAij/dxi (xj - xi)^2]
% by computing the sum for xi, xj on a grid and then using interpolation:
%   approx.= N*sum_j [-Aij(xi - xj) - 0.5 dAij/dxi (xj - xi)^2] * p(xj)dxj

% Aij*N
AijN = exp(-0.5*dXij.^2/s^2)/sqrt(2*pi*s^2);

% dAij/dxi * N
dAijN = -AijN.*dXij/s^2;

% Compute social component on a grid, then use linear interpolation
for t = 1:length(idxT)-1
    P = kron(p(:,t),ones(1,size(p,1)));
    Z = (-AijN.*dXij - 0.5*dAijN .* dXij.^2) .*  P * (x(2)-x(1));
    Z = sum(Z,1);
    Xsoc(idxT(t)+1:idxT(t+1)) = interp1(x, Z, X(idxT(t)+1:idxT(t+1)),...
        'linear', 0);
end

Xtemp = [-X, ones(length(X),1), Xsoc];
b = (Xtemp'*Xtemp)\(Xtemp'*A);
r = A - Xtemp*b;

S = sqrt( mean(r.^2) );
end