%% fitBMIdistn.m: Fit non-social, social, log-norm, and skew-norm dist'n
% Use maximiation of log-likelihood function.
%
% Generate Fig. 3 plot (for NU data)
%   - BMI distributions for year 2011,
%   - Error curves for all years
%
% Generate Supplemental Fig.
%   - S1: (for BRFSS and NHANES data)
%       - BMI distributions for year 2011,
%       - Error curves for all years
%   - S2: Parameters for nonsocial model
%
% Compute AIC criterion (Table 1)
%
% Inputs:
%   - xx = edges of bins for histogram/empirical BMI distribution
%   - BMI1 (NHANES), BMI2 (NU), BMI3 (BRFSS):
%       - 2 column array.
%       - column 1: year
%       - column 2: BMI measurement
%
% Output:
%   - out1 (NHANES), out2 (NU), out3 (BRFSS): RMS Error, AIC, fitted param.
%       - out#(:,1) = Year
%       - out#(:,2:5) = RMSE for non-social, social, log-norm, skew-norm.
%       - out#(:,6:9) = exp((AIC_min - AIC)/2)
%       - out#(:,10:11) = fitted (k0,x*) from non-social model
%       - out#(:,12:15) = fitted (k0,x*,k_S/k_b,sigma) from social model
%       - out#(:,16:17) = fitted (mu,sigma) from log-normal model
%       - out#(:,18:20) = fitted (xi, omega, alpha) from skew-norma model
%   - pdf3: fitted non-social model for BRFSS
%
% Global Variables:
%   - BMI: empirical BMI measurements
%   - x: centers of bins for empirical BMI distribution
%   - model: = 1 for non-social, 2 for social, 3 for log-normal, 4 for
%       skew-normal distribution

function [out1, out2, out3, pdf3] = fitBMIdistn(xx, BMI1, BMI2, BMI3)
global BMI x model

x = (xx(1:end-1) + xx(2:end))/2;

% RMS Error, parameters for non-social model
%   - out(:,1) = Year
%   - out(:,2:5) = RMS Error for non-social, social, log-norm,  skew-norm.
%   - out(:,6:9) = exp((AIC_min - AIC)/2)
%   - out(:,10:11) = fitted (k0,x*) from non-social model
%   - out(:,12:15) = fitted (k0,x*,k_S/k_b,sigma) from social model
%   - out(:,16:17) = fitted (mu,sigma) from log-normal model
%   - out(:,18:20) = fitted (xi, omega, alpha) from skew-norma model
out1 = sort(unique(BMI1(:,1)));
out1 = [out1, zeros(length(out1),19)];

out2 = sort(unique(BMI2(:,1)));
out2 = [out2, zeros(length(out2),19)];

out3 = sort(unique(BMI3(:,1)));
out3 = [out3, zeros(length(out3),19)];
pdf3 = zeros(length(x),length(out3));

% Perform fit
% i = 1: NHANES, i = 2: NU, i = 3: BRFSS
for i = 1:3
    % Select years T
    if i == 1
        T = out1(:,1);
    elseif i == 2
        T = out2(:,1);
    else
        T = out3(:,1);
    end
    
    for t = T'
        % Select BMI data BMI
        if i ==1
            BMI = BMI1;
        elseif i == 2
            BMI = BMI2;
        else
            BMI = BMI3;
        end
        
        % Empirical BMI distribution
        idx = BMI(:,1) == t;
        BMI = BMI(idx,2);
        
        Y = histcounts(BMI, xx, 'Normalization', 'pdf');
        
        % Fit non-social, social, log-normal, skew-normal dist'ns
        for j = 1:4
            % Select model
            model = j;
            
            % Initial guess for parameters
            if j == 1
                p0 = [10, 28];
            elseif j == 2
                p0 = [10, 28, 250, 3];
            elseif j == 3
                p0 = [3.25, 0.2];
            else
                p0 = [20, 9, 4];
            end
            
            % Optimization
            p = fminsearch(@fitBMIObj, p0, optimset('MaxFunEvals', 10000));
            
            [S, f] = fitBMIObj(p);
            
            if i == 3 && j == 1
                pdf3(:,t-T(1)+1) = f;
            end
            
            % Record RMS error and parameters from non-social model
            if i == 1
                idxt = out1(:,1) == t;
                out1(idxt, 1+j) = norm((Y-f)/length(Y),2);
                out1(idxt, 5+j) = 2*length(p) + 2*S;
                
                if j == 1
                    out1(idxt, 10:11) = p;
                elseif j == 2
                    out1(idxt, 12:15) = p;
                elseif j == 3
                    out1(idxt, 16:17) = p;
                else
                    out1(idxt, 18:20) = p;
                end
            elseif i == 2
                idxt = out2(:,1) == t;
                out2(idxt, 1+j) = norm((Y-f)/length(Y),2);
                out2(idxt, 5+j) = 2*length(p) + 2*S;
                
                if j == 1
                    out2(idxt, 10:11) = p;
                elseif j == 2
                    out2(idxt, 12:15) = p;
                elseif j == 3
                    out2(idxt, 16:17) = p;
                else
                    out2(idxt, 18:20) = p;
                end
            else
                idxt = out3(:,1) == t;
                out3(idxt, 1+j) = norm((Y-f)/length(Y),2);
                out3(idxt, 5+j) = 2*length(p) + 2*S;
                
                if j == 1
                    out3(idxt, 10:11) = p;
                elseif j == 2
                    out3(idxt, 12:15) = p;
                elseif j == 3
                    out3(idxt, 16:17) = p;
                else
                    out3(idxt, 18:20) = p;
                end
            end
        end
    end
    
    % Compute relative likelihood exp((min(AIC)-AIC)/2)
    if i == 1
        minAIC = min(out1(:,6:9),[],2);
        out1(:,6:9) = exp( ( kron(minAIC,ones(1,4)) - out1(:,6:9) )/2 );
    elseif i == 2
        minAIC = min(out2(:,6:9),[],2);
        out2(:,6:9) = exp( ( kron(minAIC,ones(1,4)) - out2(:,6:9) )/2 );
    else
        minAIC = min(out3(:,6:9),[],2);
        out3(:,6:9) = exp( ( kron(minAIC,ones(1,4)) - out3(:,6:9) )/2 );
    end
end

% Make Fig. 3: 2011 dist'n
F3(x, out1, out2, out3, BMI1, BMI2, BMI3)
end

%% Objective function for maximum likelihood parameter estimation
function [S,f0] = fitBMIObj(p)
global BMI x
% Coarse Grid (what we report)
dx0 = x(2)-x(1);
x0 = x;
f0 = fitBMIDistn(x0, p);

% Fine Grid (what we use to do computations)
dx1 = dx0/2;
x1 = x(1)-dx0/2:dx1:x(end)+dx0/2;

f1 = fitBMIDistn(x1, p);

S = interp1(x1, f1, BMI);

S = -sum(log(S));
end

%% Distribution fucntion evaluation
function f = fitBMIDistn(x, p)
global model
% Compute distribution for the model with parameters p on grid x
% model = 1: p = [k0, x*]
% model = 2: p = [k0, x*, k_S/k_b, sigma]
% model = 3: p = [mu, sigma]
% model = 4: p = [xi, omega, alpha]

dx = x(2)-x(1);

if model == 1
    % non-social model
    f = x.^(-2*(p(1)+1)) .* exp(-2*p(1)*p(2)./x);
elseif model == 2
    % social model
    % compute distribution iteratively with tolerance tol = 10^(-12)
    
    X = kron(x, ones(length(x),1));
    
    f0 = x.^(-2*(p(1)+1)) .* exp(-2*p(1)*p(2)./x);
    f0 = f0/sum(f0*dx);
    
    f = f0;
    
    l2 = 1;
    
    while l2 >= 1e-12
        f1 = f;
        
        s = exp(-0.5/p(4)^2*(X-X').^2) .* (X-X') .* ...
            (1-0.5/p(4)^2.*(X-X').^2) .* kron(f1, ones(length(x),1)) ./ ...
            X'.^2;
        
        s = s/sqrt(2*pi*p(4)^2)*dx^2;
        
        s = sum(s,2);
        
        s = (cumsum(s) - 0.5*s)';
        
        f = f0 .* exp(2*p(3)*s);
        f = f/sum(f*dx);
        
        l2 = norm((f-f1)/length(x),2);
    end
elseif model == 3
    % Log-normal distribution
    f = 1/p(2)/sqrt(2*pi)./x.*exp(-0.5/p(2)^2*(log(x)-p(1)).^2);
elseif model == 4
    % Skew-normal distribution
    f = 1/p(2)/sqrt(2*pi) * exp(-0.5*( (x - p(1))/p(2) ).^2) .*...
        (1 - erf(-p(3)*(x - p(1))/p(2)/sqrt(2)));
end

% Normalization, i.e. pdf integrates to 1
f = f/sum(f*dx);
end

%% Fig. 3 (NU data, 4x1) and supplemental Fig. S1 (NHANES, BRFSS data)
function F3(x, out1, out2, out3, BMI1, BMI2, BMI3)
global model

% Figure 3 (NU data from 2011)
figure;
color = {'-r','--r','-.b',':g'};
set(gcf, 'units', 'centimeters', 'pos', [2, 2, 8.9, 17])

% Top three panels:
%   - BMI distn
%   - log(BMI distn)
%   - difference between BMI distn and log normal distn
idx = out2(:,1) == 2011;

BMI = BMI2;
out = out2;

dx = x(2)-x(1);
Y = histcounts(BMI(BMI(:,1) == 2011,2), [x-dx/2, x(end)+dx/2],...
    'Normalization', 'pdf');

model = 1; [~,fns] = fitBMIObj(out(idx,10:11));
model = 2; [~, fs] = fitBMIObj(out(idx,12:15));
model = 3; [~, flog] = fitBMIObj(out(idx,16:17));
model = 4; [~, fskew] = fitBMIObj(out(idx,18:20));

for i = 1:3
    if i == 1
        subplot(4,1,1)
        set(gca, 'units', 'centimeters', 'pos', [2, 13.25, 6.4, 3.25])
        
        ylabel('BMI distn', 'Interpreter', 'Latex', 'FontSize', 12)
        
        axis([10, 60, 0, 0.1])
        set(gca,'XTickLabel',[])
    elseif i == 2
        subplot(4,1,2)
        set(gca, 'units', 'centimeters', 'pos', [2, 9.5, 6.4, 3.25])
        
        ylabel('BMI distn', 'Interpreter', 'Latex', 'FontSize', 12)
        
        axis([10, 60, 1e-10, 1])
        set(gca,'yscale','log')
        set(gca,'XTickLabel',[])
    else
        subplot(4,1,3)
        set(gca, 'units', 'centimeters', 'pos', [2, 5.75, 6.4, 3.25])
        
        xlabel('BMI $(x)$', 'Interpreter', 'Latex', 'FontSize', 12)
        ylabel('BMI distn$-f_{\log}(x)$', 'Interpreter', 'Latex',...
            'FontSize', 12)
        
        axis([10, 60, -0.01, 0.015])
        set(gca,'XTickLabel',{'10','20','30','40','50','60'})
        set(gca,'YTick', -0.01:0.005:0.015)
        set(gca,'YTickLabel',{'-0.01','','0','','0.01',''})
    end
    
    hold on
    box on
    
    set(gca,'XTick', 10:10:60)
    
    
    plot(x, fns - flog*(i==3), color{1}, 'LineWidth', 1)
    plot(x, fs - flog*(i==3), color{2}, 'LineWidth', 2)
    plot(x, flog - flog*(i==3), color{3})
%     plot(x, fskew - flog*(i==3), color{4}, 'LineWidth', 2)
    scatter(x, Y - flog*(i==3), '.k')
end

% Annotation
annotation('doublearrow', 'Units', 'centimeters',...
    'X', [2.75, 3.875], 'Y', [14.5, 14.5], 'Color', 'red',...
    'LineWidth',2)
annotation('doublearrow', 'Units', 'centimeters',...
    'X', [5.375, 4.25], 'Y', [14.5, 14.5], 'Color', 'red',...
    'LineWidth',2)
annotation('textbox', 'Units', 'centimeters',...
    'Position', [2, 14.125, 4, 1],...
    'String', 'diff.',...
    'FontWeight', 'bold', 'Color', 'red', 'LineStyle', 'none')
annotation('textbox', 'Units', 'centimeters',...
    'Position', [4.75, 14.125, 4, 1],...
    'String', 'diffusion',...
    'FontWeight', 'bold', 'Color', 'red', 'LineStyle', 'none')

annotation('arrow', 'Units', 'centimeters',...
    'X', [2.75, 3.5], 'Y', [15.5, 15.5], 'Color', 'blue',...
    'LineWidth',2)
annotation('arrow', 'Units', 'centimeters',...
    'X', [5.375, 4.375], 'Y', [15.5, 15.5], 'Color', 'blue',...
    'LineWidth',2)
annotation('textbox', 'Units', 'centimeters',...
    'Position', [2, 15.125, 4, 1],...
    'String', 'drift',...
    'FontWeight', 'bold', 'Color', 'blue', 'LineStyle', 'none')
annotation('textbox', 'Units', 'centimeters',...
    'Position', [4.75, 15.125, 4, 1],...
    'String', 'drift',...
    'FontWeight', 'bold', 'Color', 'blue', 'LineStyle', 'none')

% Bottom panel:
%   - RMS Error
subplot(4,1,4)
hold on;
box on;

for i = [1,3]
    plot(out(:,1), out(:,1+i), color{i})
end

for i = 2% [2,4]
    plot(out(:,1), out(:,1+i), color{i}, 'LineWidth', 2)
end

xlabel('Year', 'Interpreter', 'Latex', 'FontSize', 12)
ylabel('RMS Err. ($\times10^4$)', 'Interpreter', 'Latex', 'FontSize', 12)

axis([1995, 2015, 1e-4, 6e-4])

set(gca,'XTick', 1995:10:2015)
set(gca,'XTickLabel',{'1995','2005','2015'})

set(gca,'YTick', 1e-4:1e-4:6e-4)
set(gca,'YTickLabel',{'1','','3','','5',''})

set(gca, 'units', 'centimeters', 'pos', [2, 1.25, 6.4, 3.25])

saveas(gcf,'Fig3.fig')
% save2pdf('Fig3',gcf,600)

close

% Supplemental Figure S1 (NHANES, BRFSS data from 2011)
figure
set(gcf, 'units', 'centimeters', 'pos', [2, 2, 14.5, 17])

for j = 1:2
    % j == 1: NHANES
    % j == 2: BRFSS
    if j == 1
        idx = out1(:,1) == 2011;
        BMI = BMI1;
        out = out1;
    else
        idx = out3(:,1) == 2011;
        BMI = BMI3;
        out = out3;
    end
    
    Y = histcounts(BMI(BMI(:,1)==2011,2), [x-dx/2, x(end)+dx/2],...
        'Normalization', 'pdf');
    
    model = 1; [~,fns] = fitBMIObj(out(idx,10:11));
    model = 2; [~, fs] = fitBMIObj(out(idx,12:15));
    model = 3; [~, flog] = fitBMIObj(out(idx,16:17));
    model = 4; [~, fskew] = fitBMIObj(out(idx,18:20));
    
    % Top three panels:
    %   - BMI distn
    %   - log(BMI distn)
    %   - difference between BMI distn and log normal distn
    for i = 1:3
        if i == 1
            if j == 1
                subplot(4,2,1)
                set(gca, 'units', 'centimeters', 'pos', [2, 13, 5.5, 3.25])
                
                title('NHANES', 'FontSize', 14)
                ylabel('BMI distn', 'Interpreter', 'Latex', 'FontSize', 12)
            else
                subplot(4,2,2)
                set(gca, 'units', 'centimeters', 'pos', [8.5,13,5.5,3.25])
                
                title('BRFSS', 'FontSize', 14)
                set(gca,'YTickLabel',[])
            end
            
            axis([10, 60, 0, 0.1])
            set(gca,'XTickLabel',[])
        elseif i == 2
            if j == 1
                subplot(4,2,3)
                set(gca, 'units', 'centimeters', 'pos', [2,9.25,5.5,3.25])
                
                ylabel('BMI distn', 'Interpreter', 'Latex', 'FontSize', 12)
            else
                subplot(4,2,4)
                set(gca, 'units', 'centimeters', 'pos',[8.5,9.25,5.5,3.25])
                
                set(gca,'YTickLabel',[])
            end
            
            axis([10, 60, 1e-10, 1])
            set(gca,'yscale','log')
            set(gca,'XTickLabel',[])
        else
            if j == 1
                subplot(4,2,5)
                set(gca, 'units', 'centimeters', 'pos', [2,5.5,5.5,3.25])
                
                ylabel('BMI distn$-f_{\log}(x)$', 'Interpreter',...
                    'Latex', 'FontSize', 12)
                set(gca,'YTickLabel',{'-0.01','','0','','0.01',''})
            else
                subplot(4,2,6)
                set(gca, 'units', 'centimeters', 'pos', [8.5,5.5,5.5,3.25])
                
                set(gca,'YTickLabel',[])
            end
            
            xlabel('BMI $(x)$', 'Interpreter', 'Latex', 'FontSize', 12)
            
            axis([10, 60, -0.01, 0.015])
            set(gca,'XTickLabel',{'10','20','30','40','50','60'})
            set(gca,'YTick', -0.01:0.005:0.015)
        end
        
        hold on
        box on
        
        set(gca,'XTick', 10:10:60)
        
        plot(x, fns - flog*(i==3), color{1})
        plot(x, fs - flog*(i==3), color{2}, 'LineWidth', 2)
        plot(x, flog - flog*(i==3), color{3})
%         plot(x, fskew - flog*(i==3), color{4}, 'LineWidth', 2)
        scatter(x, Y - flog*(i==3), '.', 'MarkerEdgeColor', 'k')
    end
    
    % Bottom panel:
    %   - RMS Error
    if j == 1
        subplot(4,2,7)
        ylabel('RMS Err. ($\times10^4$)', 'Interpreter', 'Latex',...
            'FontSize', 12)
        
        axis([1995, 2015, 1e-4, 5e-4])
        
        set(gca,'XTick', 1995:10:2015)
        set(gca,'XTickLabel',{'1995','2005','2015'})
    else
        subplot(4,2,8)
        axis([1985, 2015, 1e-4, 5e-4])
        
        set(gca,'XTick', 1985:10:2015)
        set(gca,'XTickLabel',{'1985','1995','2005','2015'})
    end
    
    hold on;
    box on;
    
    for i = 1:3
        plot(out(:,1), out(:,1+i), color{i})
    end
    
%     plot(out(:,1), out(:,5), color{4}, 'LineWidth', 2)
    
    xlabel('Year', 'Interpreter', 'Latex', 'FontSize', 12)
    
    set(gca,'YTick', 1e-4:1e-4:5e-4)
    
    if j == 1
        set(gca, 'units', 'centimeters', 'pos', [2, 1, 5.5, 3.25])
        
        set(gca,'YTickLabel',{'1','','3','','5'})
    else
        set(gca, 'units', 'centimeters', 'pos', [8.5, 1, 5.5, 3.25])
        
        set(gca,'YTickLabel',[])
    end
end

saveas(gcf,'FigS1.fig')
% save2pdf('FigS1',gcf,600)

close
end